@@ -21,9 +21,9 @@ Explosion::Explosion( FixedPoint Fxp, FixedPoint Fyp, int size, int dur,
 	for(i=0; i<Vsize; i++)
 		{
 		Vbit[i].setFpos(Fxp,Fyp);
-		Vbit[i].randomDir( INT2FP((rand()>>2)%7 + 3) );
+		Vbit[i].randomDir( INT2FP((rand()>>2)&63 + 24)>>4 );
 		Vbit[i].addVel( Fxv, Fyv );
-		Vbit[i].Vcolor = ((rand()&32767)>16383) ? 254 : 253;
+		Vbit[i].Vcolor = ((rand()&255)>127) ? 254 : 253;
 		}
 	}
 
