@@ -14,10 +14,10 @@
 #include <math.h>
 #include <iostream.h>
 
+#include "SBitmap.h"
+#include "RawImageLoader.h"
+#include "cfclass.h"
 #include "Keyboard.h"
-#include "bitmaps.h"		// bitmaps for asteroids, ship, etc...
-#include "titleScn.h"		// bitmap for title screen
-#include "backScn.h"		// bitmap for high scores backdrop
 #include "Timer.h"
 #include "sasteroids.h"
 #include "HiScore.h"
@@ -27,8 +27,9 @@
 #include "Explosion.h"
 #include "Joystick.h"
 
-// not really needed yet, but harmless (and useful for testing)
-#include "Sound.h"
+
+// not ready yet
+//#include "Sound.h"
 
 // Version should be defined and passed in from Makefile
 #ifndef VERSION
@@ -46,7 +47,7 @@
 // Indexes for 1st dimension of obj	
 // The order they are in is important	
 #define	AST	0
-#define	ENEMY	96
+#define	ENEMY_I	96
 #define ENEMYBUL 97
 #define	FBUL	98
 #define	LASTBUL	102
@@ -70,18 +71,17 @@
 // to work the way they do..
  
 // Masses	
-#define M_BIG	8.0
-#define M_MED	4.0
-#define M_SMALL	1.0
-#define M_SHIP	1.5
-#define M_ENEMY	1.0
-#define M_BULLET 0.1
+#define M_BIG	INT2FP(8)
+#define M_MED	INT2FP(4)
+#define M_SMALL	INT2FP(1)
+#define M_SHIP	INT2FP(5)>>2
+#define M_ENEMY	INT2FP(1)
+#define M_BULLET INT2FP(1)>>3
 
 #define START_DIST 70		// when starting a level, place new asteroids
 				// at least this far from your ship.
 
 
-typedef struct {double x, y, mag;} Vector;
 
 /////////////////////////////////////////////////////////////////////////////
 // Global variables:	
@@ -95,6 +95,12 @@ ObjPtr	objShelf[SHIP+1];
 // The player ship is an instance of the ship object, eh?
 Ship	playerShip( SHIPSHAPE, M_SHIP );
 
+// A cfc object provides an interface to '.cf' files
+cfc	Gcf;
+
+// some bitmaps needed on-the-fly through the game
+SBitmap Gbit[8];
+SBitmap Gbackdrop;
 
 // I've got to finish C++'ing this program and hide the rest
 // of these global vars.
@@ -103,56 +109,94 @@ int	score,
 	nextbul = FBUL,			// Obj# of next bullet fired	
 	numasts, oldscore;
 
+// Glevel increments everytime you clear out a screen of asteroids
+// LEVEL increments up to 20, then no higher (as many parts of the game
+// get tougher until level 20, then stay the same after that)
 #define MAXLEVEL 20
 #define LEVEL   (Glevel <? MAXLEVEL )
 
 ////////////////////////////////////////////////////////////////////////////
 
-// ugly hack hack hack.  for convience, define white and red colors
-// into the bitmap's color palette.
+// ugly hack hack hack.  for convenience, define white and red colors
+// into the color palette.
 #define WHITE 255
 #define RED   254
 #define RED2  253
-void setBPcolormap() {
-    bitmaps.color[RED].red = 63;
-    bitmaps.color[RED].green =15;
-    bitmaps.color[RED].blue = 25;
-    		
-    bitmaps.color[RED2].red = 63;
-    bitmaps.color[RED2].green = 50;
-    bitmaps.color[RED2].blue = 10;
 
-    gl_setpalette(&bitmaps);
-}
+// set palette by loading one of the game bitmaps.
+void setGamePalette()
+	{
+	RawImageLoader loader(Gcf);
 
+	if ( loader.load("SHIP0.RAW", LOAD_PALETTE) == EXIT_SUCCESS )
+		gl_setpalettecolors( 0, loader.numColors(), loader.palette() );
+	
+	gl_setpalettecolor(RED,63,15,25); 
+	gl_setpalettecolor(RED2,63,50,10);
+	gl_setpalettecolor(WHITE,63,63,63);
+	}
+
+
+// load bitmap image from 'file' into 'b'
+void loadBitmap( Bitmap &b, const char *file )
+	{
+	RawImageLoader	loader(Gcf);
+
+	if ( loader.load(file, LOAD_IMAGE) == EXIT_SUCCESS )
+		b.setMap( loader );	
+	}
+
+	
 // set up the array of game objects.
 void initObjArray()
-{	int i;
-	for (i = 0; i < ENEMY; i++)
+	{	
+	loadBitmap( Gbit[SMALLAST], "SMALLAST.RAW" );
+	loadBitmap( Gbit[MEDAST], "MEDAST.RAW" );
+	loadBitmap( Gbit[BIGAST], "BIGAST.RAW" );
+	loadBitmap( Gbit[ENEMY], "ENEMY.RAW" );
+	loadBitmap( Gbit[BULLET], "BULLET.RAW" );
+	loadBitmap( Gbit[BULLET2], "BULLET2.RAW" );
+	loadBitmap( Gbit[SPINNER], "SPINNER.RAW" );
+	loadBitmap( Gbit[SHIELD], "SHIELD.RAW" );
+#ifdef BACKDROPS
+	loadBitmap( Gbackdrop, "BACK1.RAW" );
+#endif
+
+
+#ifdef COMPILE_BITMAPS
+	Gbit[BIGAST].compile();
+	Gbit[MEDAST].compile();
+	Gbit[SMALLAST].compile();
+	Gbit[SHIELD].compile();
+#endif
+
+	int i;
+	for (i = 0; i < ENEMY_I; i++)
 		{	
-		objShelf[i] = obj[i] = new Sprite( ASTSHAPE1, M_BIG, bigast );
+		objShelf[i] = obj[i] = new Sprite( ASTSHAPE1, M_BIG, Gbit[BIGAST] );
 		}
 
 	objShelf[SHIP] = obj[SHIP] = &playerShip;
 		
 	objShelf[ENEMYBUL] = obj[ENEMYBUL] = 
-				new Sprite( ENBULSH, M_BULLET, bullet2 );
+				new Sprite( ENBULSH, M_BULLET, Gbit[BULLET2] );
 				
-	objShelf[ENEMY] = obj[ENEMY] = 
+	objShelf[ENEMY_I] = obj[ENEMY_I] = 
 				new Enemy( ENEMYSHAPE, M_ENEMY, playerShip, *obj[ENEMYBUL] );
 	
 	obj[ENEMYBUL]->die();
 	
 	for (i = FBUL; i < LASTBUL+1; i++)
 		{	
-		objShelf[i] = obj[i] = new Sprite( BULSHAPE, M_BULLET, bullet );
+		objShelf[i] = obj[i] = new Sprite( BULSHAPE, M_BULLET, Gbit[BULLET] );
 		}	
-}
+	}
 
 // create asteroids for beginning of each level/game
 // makeasts - modified from xasteroids
 void makeasts()
-{	int i;
+	{	
+	int i;
 
 	// Erase objs from last level except ship 
 	for (i = 0; i < SHIP; i++)
@@ -171,7 +215,7 @@ void makeasts()
 		
 	// make new asteroids
 	for (i = 0; i < (LEVEL/2+1); i++) // # of Asteroids
-	{
+		{
 		obj[i]->shape = ASTSHAPE1;
 
 		obj[i]->setFpos( INT2FP((rand()>>5)%WIDTH), 
@@ -181,29 +225,29 @@ void makeasts()
 		if (distance( playerShip, *obj[i] ) > START_DIST)
 			{
 			obj[i]->randomDir( INT2FP(1) );
-			obj[i]->setBitmap(bigast);
-			obj[i]->mass = M_BIG;
+			obj[i]->setBitmap(Gbit[BIGAST]);
+			obj[i]->Fmass = M_BIG;
 			obj[i]->revive();
 			}
 		else	i--;	// ast too close to player, try again
-	}
+		}
 	numasts = i;
-}
+	}
 
 
 // nextast - from xasteroids
 int nextast()	// Find next unused asteroid object
-{	
+	{	
 	register int i;
 	for (i = 0; obj[i]->isAlive(); i++);	// guaranteed to find one
 	return i;
-}
+	}
 
 
 void blastpair(int i, int j)
 // Generate random velocity vector v.	
 // Add v to object i, -v to j.			
-{
+	{
 	unsigned int c;	// for rand	
 	FixedPoint Fvx, Fvy;
 	c = (rand()>>2);
@@ -213,90 +257,62 @@ void blastpair(int i, int j)
 
 	obj[i]->addVel( Fvx, Fvy );
 	obj[j]->addVel( -Fvx, -Fvy );
-}
+	}
 
 
+typedef struct {FixedPoint x, y, mag;} FVector;
+
 // dot product of 2 vectors	
-#define dot(i,j)	(i.x*j.x + i.y*j.y)
+#define dot(i,j)	(FPMULT( i.x, j.x ) + FPMULT( i.y, j.y ))
 
-// bounce - from xasteroids
-// cause two objects to collide elastically	
-// Hasn't been converted to fixed point math yet.
 void bounce(int i, int j )
-{
-	double	temp;
-	Vector	vi, vj,		// velocity of objs i, j		
+	{
+	FixedPoint	temp;
+	FVector	vi, vj,		// velocity of objs i, j		
 		ij,		// vector from center of i to center of j 
 		vi_ij, vj_ij,	// velocity of obj along vector ij	
 		vipij, vjpij,	// velocity perpendicular to ij		
 		vi_ijf, vj_ijf;	// post-collision velocity along ij	
 
-	vi.x = obj[i]->xvel(); vi.y = obj[i]->yvel();
-	vj.x = obj[j]->xvel(); vj.y = obj[j]->yvel();
-	ij.x = obj[j]->x() - obj[i]->x(); ij.y = obj[j]->y() - obj[i]->y();
-	ij.mag = sqrt(ij.x*ij.x + ij.y*ij.y);
-
-/*
-Calculate velocities projected onto ij;
-vi_ij = vi*cos(a) = (vi dot ij) / |ij|		
-*/
-
+	vi.x = obj[i]->VFxvel; vi.y = obj[i]->VFyvel;
+	vj.x = obj[j]->VFxvel; vj.y = obj[j]->VFyvel;
+	ij.x = obj[j]->VFx - obj[i]->VFx; ij.y = obj[j]->VFy - obj[i]->VFy;
+	ij.mag = FastMath::intSqrt( FP2INT(ij.x) * FP2INT(ij.x) + 
+	                    FP2INT(ij.y) * FP2INT(ij.y) );
+	
 	vi_ij.mag = dot(vi, ij) / ij.mag;
-	vi_ij.x = (ij.x * vi_ij.mag) / ij.mag;
-	vi_ij.y = (ij.y * vi_ij.mag) / ij.mag;
+	vi_ij.x = FPMULT(ij.x , vi_ij.mag) / ij.mag ;
+	vi_ij.y = FPMULT(ij.y , vi_ij.mag) / ij.mag ;
+	
 	vj_ij.mag = dot(vj, ij) / ij.mag;
-	vj_ij.x = (ij.x * vj_ij.mag) / ij.mag;
-	vj_ij.y = (ij.y * vj_ij.mag) / ij.mag;
+	vj_ij.x = FPMULT(ij.x , vj_ij.mag) / ij.mag ;
+	vj_ij.y = FPMULT(ij.y , vj_ij.mag) / ij.mag ;
 
 	if (vi_ij.mag - vj_ij.mag < 0)	
 		return;
-/* 
-Objs moving away from each other -
-Since objs are round (at least when bouncing), this means
-they are moving away from each other already.
-*/
 
-/*
-Calculate component of velocities perpendicular to ij:
-	|vipij| = |vi|*sin(a) = |vi x ij| / |ij|
-Same as
-	|vipij| = |vi|*cos(M_PI/2 - a) = (vi dot (perp. to ij)) / |ij|
-*/
+	temp = (FPMULT( vi.y, ij.x) - FPMULT( vi.x, ij.y)) / (ij.mag*ij.mag);
+	vipij.x = -FPMULT(ij.y,temp); vipij.y = FPMULT(ij.x,temp);
+	
+	temp = (FPMULT( vj.x, ij.y) - FPMULT( vj.y, ij.x)) / (ij.mag*ij.mag);
+	vjpij.x = -FPMULT(ij.y,temp); vjpij.y = FPMULT(ij.x,temp);
 
-	temp = vi.y*ij.x - vi.x*ij.y;	// - (X-product when 3rd coord is 0)
-	temp /= (ij.mag*ij.mag);
-	vipij.x = -ij.y*temp; vipij.y = ij.x*temp;
-	temp = (vj.x*ij.y - vj.y*ij.x) / (ij.mag*ij.mag);
-	vjpij.x = -ij.y*temp; vjpij.y = ij.x*temp;
-
-/*
-Calculate the linear elastic collision along ij:
-	mass(i)*vi_ij + mass(j)*vj_ij = mass(i)*vi_ijf + mass(j)*vj_ijf
-	vi_ij + vi_ijf = vj_ij + vj_ijf	(derived by dividing equation
-	for conservation of kinetic energy
-	 by eq. for cons. of momentum)
-*/
-
-	temp = obj[i]->mass/obj[j]->mass;
-	vj_ijf.x = (temp * (2*vi_ij.x - vj_ij.x) + vj_ij.x) / (1 + temp);
-	vj_ijf.y = (temp * (2*vi_ij.y - vj_ij.y) + vj_ij.y) / (1 + temp);
+	temp = FPDIV(obj[i]->Fmass,obj[j]->Fmass);
+	vj_ijf.x = FPDIV( FPMULT( temp, (2*vi_ij.x - vj_ij.x)) + vj_ij.x, INT2FP(1) + temp);
+	vj_ijf.y = FPDIV( FPMULT( temp, (2*vi_ij.y - vj_ij.y)) + vj_ij.y, INT2FP(1) + temp);
 	vi_ijf.x = vj_ijf.x + vj_ij.x - vi_ij.x;
 	vi_ijf.y = vj_ijf.y + vj_ij.y - vi_ij.y;
 
-/*
-Now, given vi_ijf and vj_ijf, add them to the perpendicular
-	components to get final velocity vectors
-*/
+	obj[i]->setFvel( vi_ijf.x + vipij.x, vi_ijf.y + vipij.y );
+	obj[j]->setFvel( vj_ijf.x + vjpij.x, vj_ijf.y + vjpij.y );
 
-	obj[i]->setFvel( FLOAT2FP(vi_ijf.x + vipij.x), FLOAT2FP(vi_ijf.y + vipij.y) );
-	obj[j]->setFvel( FLOAT2FP(vj_ijf.x + vjpij.x), FLOAT2FP(vj_ijf.y + vjpij.y) );
+	}
 
-}
 
 
 // botline - modified from xasteroids
 void botline() 
-{	
+	{	
 	char text[70];
 	int	y;
 	
@@ -311,10 +327,10 @@ void botline()
 		sprintf(text, "%07d Ships:%2d  Lvl:%2d  Shield:", 
 	               score, playerShip.ships(), Glevel );
 		
-		gl_line( 262, y+3, 292   ,y+3, 254);
-		gl_line( 262, y+4, 262+l ,y+4, 255);
-		gl_line( 262, y+5, 262+l ,y+5, 255);
-		gl_line( 262, y+6, 292   ,y+6, 254);
+		gl_hline( 262, y+3, 292   , 254);
+		gl_hline( 262, y+4, 262+l , 255);
+		gl_hline( 262, y+5, 262+l , 255);
+		gl_hline( 262, y+6, 292   , 254);
 		}
 	else	{
 		sprintf(text, "%07d Ships:%2d  Lvl:%2d  Shield:%2d", 
@@ -324,14 +340,12 @@ void botline()
 
 	gl_write(0, y, text);
 	
+	}
 
-}
-
-void printss() {};
 
 // upscore - from xasteroids
 void upscore(int killer, int up)	// Only award score for things the player shot 
-{	
+	{	
 	if (killer != ENEMYBUL && killer != SHIP)
 		score = score + up;
 
@@ -341,13 +355,13 @@ void upscore(int killer, int up)	// Only award score for things the player shot
 		playerShip.addShip();
 		oldscore = score;
 		}
-}
+	}
 
 
 // killast - modified from xasteroids
 void killast(int killer, int i)
 	// i = Asteroid # to kill	
-{	
+	{	
 	int k=0, na=0, oldna=0;
 
 	if (obj[i]->shape == ASTSHAPE1)
@@ -358,17 +372,17 @@ void killast(int killer, int i)
 		
 		na = nextast();
 		obj[na]->setFpos( obj[i]->VFx, obj[i]->VFy );
-		obj[na]->setFvel( obj[i]->VFxvel, obj[i]->VFyvel );
+		obj[na]->setFvel( obj[i]->VFxvel>>1, obj[i]->VFyvel>>1 );
 
 		obj[na]->revive();
 		obj[na]->shape = ASTSHAPE2;
-		obj[na]->setBitmap(medast);
-		obj[na]->mass = M_MED;
+		obj[na]->setBitmap(Gbit[MEDAST]);
+		obj[na]->Fmass = M_MED;
 		
 	// every once in a while, one of the medium asteroids is replaced
 	// with a little devil ship that flies around and
 	// tries to crash into you..
-		if (((rand()>>4)%60)<Glevel)
+		if (((rand()>>4)%100)<Glevel)
 			{
 			obj[i] = new Spinner( SPINNERSHAPE, M_BIG, playerShip );
 			((Spinner *)obj[i])->newShip();
@@ -377,8 +391,8 @@ void killast(int killer, int i)
 		else
 			{
 			obj[i]->shape = ASTSHAPE2;
-			obj[i]->setBitmap(medast);
-			obj[i]->mass = M_MED;
+			obj[i]->setBitmap(Gbit[MEDAST]);
+			obj[i]->Fmass = M_MED;
 			numasts++;
 			}
 		blastpair(i, na);
@@ -397,11 +411,11 @@ void killast(int killer, int i)
 
 			obj[na]->revive();
 			obj[na]->shape = ASTSHAPE3;
-			obj[na]->setBitmap(smallast);
-			obj[na]->mass = M_SMALL;
+			obj[na]->setBitmap(Gbit[SMALLAST]);
+			obj[na]->Fmass = M_SMALL;
 			if (k == 1) blastpair(oldna,na);
 			}
-		if (((rand()>>4)%120)<Glevel)
+		if (((rand()>>4)&511)<(Glevel+10))
 			{
 			obj[i] = new Spinner( SPINNERSHAPE, M_BIG, playerShip );
 			((Spinner *)obj[i])->newShip();
@@ -410,8 +424,8 @@ void killast(int killer, int i)
 		else
 			{
 			obj[i]->shape = ASTSHAPE3;
-			obj[i]->setBitmap(smallast);
-			obj[i]->mass = M_SMALL;
+			obj[i]->setBitmap(Gbit[SMALLAST]);
+			obj[i]->Fmass = M_SMALL;
 			numasts++;
 			}
 
@@ -440,12 +454,11 @@ void killast(int killer, int i)
 		}
 	else	// enemy {ship or bullet}	
 		{
-		Sound::play_file("boom.voc");
+		//Sound::play_file("boom.voc");
 		obj[i]->explode(9, 7);
 		obj[i]->die(); upscore(killer, 700);
 		}
-}
-
+	}
 
 
 //	Draw objects
@@ -480,7 +493,12 @@ int moveobjs()
 	for (i = 0; i < FBUL; i++)
 	    if (obj[i]->isAlive())
 	        {
-		if (playerShip.isAlive() && collide( *obj[i], playerShip))
+	        if (i!=ENEMY_I && i!=ENEMYBUL && obj[ENEMY_I]->isAlive() 
+	            && collide( *obj[i], *obj[ENEMY_I]) )
+	            	{
+	            	bounce(ENEMY_I, i);
+	            	}
+	        else if (playerShip.isAlive() && collide( *obj[i], playerShip))
 			{	
 			if (playerShip.shielded()) 
 				{
@@ -490,7 +508,7 @@ int moveobjs()
 			else
 				{	
 				crash = 2;
-				Sound::play_file("crash.voc");
+				//Sound::play_file("crash.voc");
 				playerShip.explode(BMAX-1, 50);
 				playerShip.die();
 				killast(SHIP, i);
@@ -501,7 +519,7 @@ int moveobjs()
 			}
 		for (j = ENEMYBUL; j < LASTBUL+1; j++)
 			if (obj[j]->isAlive() && collide( *obj[i], *obj[j]) &&
-			   (j != ENEMYBUL || (i != ENEMYBUL && i != ENEMY)))
+			   (j != ENEMYBUL || (i != ENEMYBUL && i != ENEMY_I)))
 				{	
 				obj[j]->die();	// Kill the bullet 
 				// Don't have 2 bullets kill same asteroid
@@ -531,7 +549,7 @@ void fire()
 	obj[nextbul]->setTime(30);	// loops before bullet expires	
 	nextbul++; if (nextbul == LASTBUL+1) nextbul = FBUL;
 
-	Sound::play_file("shotgun.voc");
+	//Sound::play_file("shotgun.voc");
 	}
 
 // return 1=easy, 4=medium, 8=hard
@@ -564,64 +582,89 @@ int selectGame()
 return 0;
 }
 
-void showInfo() 
-{
-Ui::drawToPhysical();
-gl_clearscreen(0);
-gl_setpalette(&backScn);
-back.put(0,0);
+void displayScreen(const char *file)
+	{
+	RawImageLoader	loader(Gcf);
 
-gl_setwritemode(WRITEMODE_MASKED);
+	loader.load(file, LOAD_IMAGE|LOAD_PALETTE);
+	
+	// assign TITLE.RAW to a bitmap
+	SBitmap scn;
+	scn.setMap( loader );
 
-gl_write(120,0,"SASTEROIDS");
+	// set palette according to TITLE.RAW palette
+	gl_setpalettecolors( 0, loader.numColors(), loader.palette() );
+	loader.releasePaletteOwnership();
+	
+	// force 255 to white for text
+	gl_setpalettecolor(WHITE,63,63,63);
 
-gl_write(10,20, "TO PLAY THE GAME");
-gl_write(10,30, "");
-gl_write(10,40, "  KEY        ACTION");
-gl_write(10,50, " ----------  -----------------------");
-gl_write(10,60, " LEFT ARROW  TURN SHIP LEFT");
-gl_write(10,70, " RIGHT ARRW  TURN SHIP RIGHT");
-gl_write(10,80, " LEFT CTRL   FIRE");
-gl_write(10,90, " UP ARROW    THRUST");
-gl_write(10,100," DOWN ARROW  SHIELD, ONE PER SHIP");
-gl_write(10,110," LEFT ALT    HYPERSPACE (ZAP TO");
-gl_write(10,120,"             RANDOM AREA ON SCREEN)");
-gl_write(10,130," P           PAUSE");
-gl_write(10,140," Q           QUIT GAME");
-gl_write(6,160,"Send bug reports, patches, sounds");
-gl_write(6,170,"or cool bitmaps to pitzel@cs.sfu.ca");
-
-gl_write(104,190,"(Press any key)");
-gl_setwritemode(WRITEMODE_OVERWRITE);
-
-};
-
-void showTitle()
-{
 	Ui::drawToPhysical();
-
+	int x = ( WIDTH - scn.width() )>>1;
+	int y = ( HEIGHT - scn.height() )>>1;
 	gl_clearscreen(0);
-	gl_setpalette(&titleScn);
-	title.put(0,0);
+	scn.put(x,y);
+
+	}
+	
+void showInfo() 
+	{
+	displayScreen("BACK.RAW");
+	
+	int x = (WIDTH - 320)>>1;
+	int y = (HEIGHT - 200)>>1;
 
 	gl_setwritemode(WRITEMODE_MASKED);
 
-	gl_write(15 , 80, "ver "VERSION );
-	gl_write(110,105,"'I' FOR INFORMATION");
-	gl_write(110,115,"'H' FOR HIGH SCORES");
-	gl_write(110,125,"'S' TO START GAME");
-	gl_write(110,135,"'Q' TO QUIT");
+	gl_write( x+120, y+0,"SASTEROIDS");
+
+	gl_write( x+10, y+20, "TO PLAY THE GAME");
+	gl_write( x+10, y+30, "");
+	gl_write( x+10, y+40, "  KEY        ACTION");
+	gl_write( x+10, y+50, " ----------  -----------------------");
+	gl_write( x+10, y+60, " LEFT ARROW  TURN SHIP LEFT");
+	gl_write( x+10, y+70, " RIGHT ARRW  TURN SHIP RIGHT");
+	gl_write( x+10, y+80, " LEFT CTRL   FIRE");
+	gl_write( x+10, y+90, " UP ARROW    THRUST");
+	gl_write( x+10, y+100," DOWN ARROW  SHIELD, ONE PER SHIP");
+	gl_write( x+10, y+110," LEFT ALT    HYPERSPACE (ZAP TO");
+	gl_write( x+10, y+120,"             RANDOM AREA ON SCREEN)");
+	gl_write( x+10, y+130," P           PAUSE");
+	gl_write( x+10, y+140," Q           QUIT GAME");
+	gl_write( x+6, y+160,"Send bug reports, patches, sounds");
+	gl_write( x+6, y+170,"or cool bitmaps to pitzel@cs.sfu.ca");
+
+	gl_write( x+104, y+190,"(Press any key)");
+	gl_setwritemode(WRITEMODE_OVERWRITE);
+
+	};
+
+void showTitle()
+{
+	displayScreen("TITLE.RAW");
+
+	int x = (WIDTH - 320)>>1;
+	int y = (HEIGHT - 200)>>1;
+	
+	gl_setwritemode(WRITEMODE_MASKED);
+
+	gl_write( x+15 , y+ 80, "ver "VERSION );
+	gl_write( x+110, y+105, "'I' FOR INFORMATION");
+	gl_write( x+110, y+115, "'H' FOR HIGH SCORES");
+	gl_write( x+110, y+125, "'S' TO START GAME");
+	gl_write( x+110, y+135, "'Q' TO QUIT");
 	gl_setwritemode(WRITEMODE_OVERWRITE);
 }
 
-
 // get name for high scores list
 String getHighScoreName()
 {
-char buf[21];
-
+char buf[31];
 for(int i=0; i<21; i++) buf[i]=' ';    
 
+// default to user's login name
+strcpy(buf,getlogin());
+
 Ui::drawToPhysical();
 
 gl_clearscreen(0);
@@ -631,7 +674,9 @@ gl_write( 20, 40, "You made the high score list.");
 
 gl_write( 20, 60, "Please enter your name :" );
 
+Ui::setFontColor(RED2);
 Ui::input( 20, 80, buf, 20 );
+Ui::setFontColor(WHITE);
 
 return String(buf);
 }
@@ -644,24 +689,24 @@ int i;
 char buf[10];
 String line;
 
-Ui::drawToPhysical();
-gl_clearscreen(0);
-gl_setpalette(&backScn);
-back.put(0,0);
+displayScreen("BACK.RAW");
 
+int x = (WIDTH - 320)>>1;
+int y = (HEIGHT - 200)>>1;
+	
 gl_setwritemode(WRITEMODE_MASKED);
-gl_write(120,0,"HIGH SCORES");
+gl_write( x+120, y+0,"HIGH SCORES");
 
 for(i=0; i<HiScoreListSize; i++)
 	{
 	sprintf(buf,"%d.",i+1);
-	gl_write(40,40+15*i,buf);
+	gl_write( x+40, y+40+15*i,buf);
 	
 	line = s[i];
-	gl_write(70,40+15*i, (char *)(const char*)line );
+	gl_write( x+70, y+40+15*i, (char *)(const char*)line );
 	}
 
-gl_write(104,190,"(Press any key)");
+gl_write( x+104, y+190,"(Press any key)");
 gl_setwritemode(WRITEMODE_OVERWRITE);
 }
 
@@ -683,18 +728,56 @@ for(j=0; j<loops; j++)
 	//	Draw objects
 	drawobjs();
 
-	gl_write(120,100,"GAME  OVER");
+	gl_write( (WIDTH>>1) - 40, HEIGHT>>1, "GAME  OVER");
 	botline();
 	
 	Ui::updateScreen();
+#ifdef BACKDROPS
+	gl_putbox(0,0,320,200,Gbackdrop.map());
+#else			
 	gl_clearscreen(0);
+#endif
 	Timer::sync();
 	}
 
 gl_setwritemode(WRITEMODE_OVERWRITE);
 }
 
+
+// Prepares keyboard for game play.
+// Puts keyboard into raw mode, and sets appropriate delays on
+// key repeat.
+int setGameKeyboard() return result;
+	{
+	result = Keyboard::init();
+		
+	if (!result)
+		{
+		keyboard_translatekeys(TRANSLATE_CURSORKEYS);
 	
+		// set delay on fire key to 1/4 second
+		Keyboard::setdelay(LEFT_CTRL,FPS/4);
+		// delay on hyperspace to 1 second
+		Keyboard::setdelay(LEFT_ALT,FPS);
+
+		Keyboard::setdelay(SCANCODE('t'),FPS); // toggle timings
+		Keyboard::setdelay(SCANCODE('p'),FPS); // pause game
+		Keyboard::setdelay(SCANCODE('s'),FPS); // new ship
+		Keyboard::setdelay(SCANCODE('q'),FPS); // quit
+	
+		// hey, there's an advantage to peeking at code :-)
+		Keyboard::setdelay(SCANCODE('b'),FPS); // secret - apply brakes
+		// this one makes for a bouncy game.	
+		Keyboard::setdelay(SCANCODE('0'),FPS); // secret - 1 hour shield
+		Keyboard::setdelay(SCANCODE('n'),FPS); // secret - advance level
+	
+		// Note: no delays set on the rotate left/right and thrust keys.
+		}
+		
+	return result;
+	}
+
+
 // play a game..
 void playGame()
 {
@@ -735,35 +818,19 @@ void playGame()
 	playerShip.reset();
 	playerShip.revive();
 	
-        setBPcolormap(); // set palette to game bitmaps' colors 
+        setGamePalette(); // hmm.. I think sets the Palette for the game
+
 
 	// all drawing takes place on the virtual screen 
 	Ui::drawToVirtual();
 	gl_clearscreen(0);
 
-	//*************************** 
-	// put keyboard into raw mode
-	if (Keyboard::init())
+	if (setGameKeyboard())
 		{
 		cerr << "Sasteroids:: cannot init keyboard" << endl;
 		return;
 		}
-		
-	keyboard_translatekeys(TRANSLATE_CURSORKEYS);
-	
-	// set delay on fire key to 1/4 second
-	Keyboard::setdelay(LEFT_CTRL,FPS/4);
-	// delay on hyperspace to 1 second
-	Keyboard::setdelay(LEFT_ALT,FPS);
 
-	Keyboard::setdelay(SCANCODE('t'),FPS);
-	Keyboard::setdelay(SCANCODE('p'),FPS);
-	Keyboard::setdelay(SCANCODE('s'),FPS);
-	Keyboard::setdelay(SCANCODE('q'),FPS);
-	Keyboard::setdelay(SCANCODE('b'),FPS);
-	Keyboard::setdelay(SCANCODE('0'),FPS);
-
-	//***************************
 	gl_setwritemode(WRITEMODE_MASKED);
 	
 	Timer::init();	// get animation timer ready
@@ -772,7 +839,7 @@ void playGame()
 	    {   
 	    if (numasts==0) 
 		{
-		if (Glevel<20) Glevel++;
+		Glevel++;
 		makeasts();
 		}
 
@@ -788,25 +855,34 @@ void playGame()
 		Keyboard::update();
 
 		if (Keyboard::keyhit(SCANCODE('p')))
-			pause = 1 - pause;
+			pause = 1;
 
 		if (Keyboard::keyhit(SCANCODE('t')))
 			showTimings = 1 - showTimings; 
 
 		if (Keyboard::keyhit(SCANCODE('q')))  // quit game
-			GameOver = 1;
+			{
+			// yesNo() wasn't written for raw keyboard mode.
+			gl_setwritemode(WRITEMODE_OVERWRITE);
+			Keyboard::restore();
+			if (Ui::yesNo("End This Game"))
+			    		{
+			    		GameOver = 1;
+			    		}
+			gl_setwritemode(WRITEMODE_MASKED);
+
+			if (setGameKeyboard())
+				{
+				cerr << "Sasteroids: couldn't re-init keyboard" << endl;
+				GameOver=1;
+				}
+			}
 
 		// cheat -- set shields to last for an hour
 		if (Keyboard::keyhit(SCANCODE('0'))) 
 			playerShip.shieldMax(FPS*3600);
 
-/*			if (Ui::yesNo("End This Game"))
-			    		{
-			    		GameOver = 1;
-			    		}
-			    	break;
-*/
-	        if (playerShip.isAlive() & !pause)
+	        if (playerShip.isAlive())
 	            {
 		    if (Joy0.ok())
 		    	{
@@ -841,6 +917,8 @@ void playGame()
 		    	playerShip.VFxvel=0;
 		    	playerShip.VFyvel=0;
 		    	}
+		    if (Keyboard::keyhit(SCANCODE('n')))
+		    		Glevel++;
 		    	
 		    if (Keyboard::keyhit(LEFT_CTRL))
 		    	fire();
@@ -848,10 +926,10 @@ void playGame()
 		    if (Keyboard::keyhit(LEFT_ALT))
 		    	playerShip.hyper();
 		    	
-		    } //player ship alive && !pause
- 
-		if (!playerShip.isAlive() & !pause)
+		    }
+		else
 		    {
+		    // !playerShip.alive()
 		    if (Joy0.ok())
 		    	{
 		    	if (Joy0.aPressed()||Joy0.bPressed())
@@ -870,31 +948,24 @@ void playGame()
 		    		}
 		     }
 
-		if (!pause)
-			{   
-			moveobjs();
+		moveobjs();
 
-		   	//if ( (playerShip.ships()) && (score>0) ) 
-		   	//	score--;	// timer effect	
-
-			if (!playerShip.ships()) 
-				{
-		   		GameOver = 1;
-		   		gameOverLoop();
-		   		}
+		if (!playerShip.ships()) 
+			{
+	 		GameOver = 1;
+	   		gameOverLoop();
+	   		}
 		   		
-			i = (rand()>>8) & 255;
-			if (!obj[ENEMY]->isAlive())
-				{   
-				if (i < Glevel)
-			    		{
-			    		c = rand();
-					if (c < Glevel<< ((numasts>10) ? 2:4) ) 
-						((Enemy*)(obj[ENEMY]))->newShip(FLOAT2FP(WIDTH/(FPS/3*(-Glevel+31))), (Glevel*2)+10);
-					}
+		i = (rand()>>8) & 255;
+		if (!obj[ENEMY_I]->isAlive())
+			{   
+			if (i < Glevel)
+		    		{
+		    		c = rand();
+				if (c < Glevel<<4 ) 
+					((Enemy*)(obj[ENEMY_I]))->newShip(FLOAT2FP(WIDTH/(FPS/3*(-LEVEL+31))), (Glevel*2)+10);
 				}
-			} // !pause
-	
+			}
 
 		if (!GameOver)
 			{
@@ -902,10 +973,9 @@ void playGame()
 
 			if (!playerShip.isAlive())
 			    	{
-				if (!pause)
-				    	gl_write( (WIDTH>>1) - 88,
-				    	           (HEIGHT>>1) + 40,
-				    	           "PRESS 's' FOR NEXT SHIP");
+				gl_write( (WIDTH>>1) - 88,
+				           (HEIGHT>>1) + 30,
+			   	           "PRESS 's' FOR NEXT SHIP");
 
 				// flash where ship will be placed
 				// once per second
@@ -917,44 +987,55 @@ void playGame()
 				}
 			}
 
-		if (pause)
-      	    		gl_write( (WIDTH>>1) - 13*8, (HEIGHT>>1) + 40,
-          	           "PRESS 'p' TO CONTINUE GAME");
-					
 		// either show frame timings or game stats on bottom line
 		if (!showTimings)
 			{
 			botline();
-			Ui::updateScreen(); 
-			gl_clearscreen(0);
+			}
+
+		if (pause)
+			{
+      	    		gl_write( (WIDTH>>1) - 13*8, (HEIGHT>>1) + 40,
+          	           "PRESS 'p' TO CONTINUE GAME");
+			Ui::updateScreen();
+			Keyboard::restore();
+
+			// wait for 'p' to unpause game			
+			while (vga_getkey()!='p')
+				{
+				//  1/10th of a second
+				usleep(100000);
+				}
+
+			if (setGameKeyboard())
+				{
+				cerr << "Sasteroids: couldn't re-init keyboard" << endl;
+				GameOver=1;
+				}
+			pause = 0;
+			}
+		else
+			Ui::updateScreen();
+
+			
+#ifdef BACKDROPS
+		memcpy(VBUF,Gbackdrop.map(),64000);
+//		gl_putbox(0,0,320,200,Gbackdrop.map());
+#else			
+		gl_clearscreen(0);
+#endif
+
+		if (!showTimings)
+			{
 			Timer::sync();   
 			}
 		else
 			{
 			// time how long to create one frame of animation
-			Ui::updateScreen(); 
-			gl_clearscreen(0);
-
 			sprintf(text,"usec per frame: %d",  Timer::sync() );
 			gl_write(0, HEIGHT-Ui::fontHeight(), text);
 			}
 
-			/* for Timing tests, ignore:
-			t1 = Timer::sync();
-			if (t1<0)  cout << t1 << " ";
-			else 
-			    	{
-			    	t2 += t1;
-				cnt++;
-				}
-				
-			if (cnt==100)
-				{
-				GameOver=1;
-				cout << "!" << t2/cnt << endl;
-				}
-			*/
-			
 		} //while (numasts)
 	} //for(level..
 Keyboard::restore();
@@ -963,6 +1044,13 @@ Keyboard::restore();
 
 int main(int argc, char **argv)
 {	
+	// a .cf file is a collection of files. In this case,
+	// sast.cf contains the bitmaps, sounds, etc used in the game
+	if (Gcf.open(BINDIR"sast.cf")!=0)
+		{
+		cerr << "Open failed on sast.cf" << endl;
+		exit(-1);
+		}
 
 	HiScore	scores("sast.scores");
 
@@ -984,22 +1072,12 @@ int main(int argc, char **argv)
 	// randomize using timer
 	srand( (unsigned int) time(NULL) );
 
-	titleScnInit();	 // define title screen bitmaps, color map
-
-	bitmapsInit();	 // define bitmaps, color map 
-	
-#ifdef COMPILE_BITMAPS
-	bigast.compile();
-	medast.compile();
-	smallast.compile();
-#endif
-	backScnInit();
+	initObjArray();	// init game objects
 
 	cout << "Generating rotated bitmaps" << endl;
 	Ship::init();	// setup up player's ship
 	Spinner::init();
 	
-	initObjArray();	// init game objects
 
 	Timer::setFrameRate( FPS );
 
@@ -1011,7 +1089,7 @@ int main(int argc, char **argv)
 
 	int done=0;
 
-	Sound::init();
+	//Sound::init();
 	
 	showTitle();
 	
@@ -1059,7 +1137,7 @@ int main(int argc, char **argv)
 
 	// shutdown & restore text mode
 	Ui::restore();
-	Sound::restore();
+	//Sound::restore();
 
 	return 0;
 }
